﻿namespace S7_PLC_Monitoring
{
    using Newtonsoft.Json;
    using S7.Net;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net;
    using System.Threading;
    using System.Windows.Forms;
    using System.Linq;
    using System.Configuration;


    /// <summary>
    /// Defines the <see cref="ConnectToPLCControl" />.
    /// </summary>
    public partial class ConnectToPLCControl : UserControl
    {
        /// <summary>
        /// Defines the json_file_path.
        /// </summary>
        internal string file_path = ConfigurationManager.AppSettings["DEFAULT_XLS_FILEPATH"];

        internal dynamic event_data;

        string json_string;

        internal List<string> plc_types = new List<string>();

        /// <summary>
        /// Initializes a new instance of the <see cref="ConnectToPLCControl"/> class.
        /// </summary>
        public ConnectToPLCControl()
        {
            InitializeComponent();
            object[] CPU_type = { CpuType.S71200, CpuType.S71500, CpuType.S7200, CpuType.S7200Smart, CpuType.S7300, CpuType.S7400, CpuType.Logo0BA8 };
            CPU_type_cbox.Items.AddRange(CPU_type);
            CPU_type_cbox.DropDownStyle = ComboBoxStyle.DropDownList;
            IP_textbox.Text = ConfigurationManager.AppSettings["DEFAULT_IP_VAL"];
            slot_no_NUD.Value = 2;
            con_failed_label.Hide();
        }

        /// <summary>
        /// The connect_button_Click_1.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void connect_button_Click_1(object sender, EventArgs e)
        {

            con_failed_label.Hide();
            string plc_ip = IP_textbox.Text;
            short rack_no = Convert.ToInt16(Math.Round(rack_no_NUD.Value, 0));
            short slot_no = Convert.ToInt16(Math.Round(slot_no_NUD.Value, 0));
            string selected_CPU_type = CPU_type_cbox.Text;
            Plc plc;
            switch (selected_CPU_type)
            {
                case "S71200":
                    plc = new Plc(CpuType.S71200, plc_ip, rack_no, slot_no);
                    break;
                case "S71500":
                    plc = new Plc(CpuType.S71500, plc_ip, rack_no, slot_no);
                    break;
                case "S7200":
                    plc = new Plc(CpuType.S7200, plc_ip, rack_no, slot_no);
                    break;
                case "S7200Smart":
                    plc = new Plc(CpuType.S7200Smart, plc_ip, rack_no, slot_no);
                    break;
                case "S7300":
                    plc = new Plc(CpuType.S7300, plc_ip, rack_no, slot_no);
                    break;
                case "S7400":
                    plc = new Plc(CpuType.S7400, plc_ip, rack_no, slot_no);
                    break;
                case "Logo0BA8":
                    plc = new Plc(CpuType.Logo0BA8, plc_ip, rack_no, slot_no);
                    break;
                default:
                    plc = new Plc(CpuType.S71500, plc_ip, rack_no, slot_no);
                    break;

            }
            System.Net.ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;

            try
            {
                plc.Open();
                if (plc.IsConnected == true)
                {
                    string plc_type = (string)plc_type_cb.SelectedItem;
                    var display_form_thread = new Thread(() => Application.Run(new DataDisplayForm(plc, file_path, plc_type)));
                    display_form_thread.SetApartmentState(ApartmentState.STA);
                    display_form_thread.Start();
                    System.Windows.Forms.Application.Exit();
                }
            }
            catch (Exception)
            {
                con_failed_label.Show();
            }
        }

        /// <summary>
        /// The select_json_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void select_json_btn_Click(object sender, EventArgs e)
        {
            try
            {
                json_file_error.Hide();

                using (OpenFileDialog openFileDialog = new OpenFileDialog())
                {
                    openFileDialog.InitialDirectory = Environment.CurrentDirectory;
                    openFileDialog.Filter = "JSON files (*json)|*.json|All files (*.*)|*.*";
                    openFileDialog.FilterIndex = 1;
                    openFileDialog.RestoreDirectory = true;

                    if (openFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        //Get the path of specified file
                        file_path = openFileDialog.FileName;

                    }
                }
                selected_json_file.Text = "Selected JSON file: " + file_path;
                selected_json_file.Show();

                json_string = File.ReadAllText(file_path);
                event_data = JsonConvert.DeserializeObject(json_string);

                foreach (var particular_event in event_data.events)
                {
                    plc_types.Add((string)particular_event.Value.plc_type);
                }
                for (int i = 0; i < plc_types.Count; i++)
                {
                    int index = plc_types[i].IndexOf(":");
                    if (index >= 0)
                        plc_types[i] = plc_types[i].Substring(index + 2);
                }
                foreach (string plc_type in plc_types.Distinct())
                {
                    plc_type_cb.Items.Add(plc_type);
                }

            }
            catch (Exception)
            {
                json_file_error.Show();
            }
        }
    }
}
