﻿namespace S7_PLC_Monitoring
{
    using Newtonsoft.Json;
    using System;
    using System.IO;
    using System.Windows.Forms;
    using S7.Net;
    using System.Collections.Generic;
    using System.Dynamic;
    using Microsoft.CSharp.RuntimeBinder;
    using System.Text.RegularExpressions;
    using System.Configuration;

    /// <summary>
    /// Defines the <see cref="DataDisplayMainControl" />.
    /// </summary>
    /// 

    public partial class DataDisplayMainControl : UserControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DataDisplayMainControl"/> class.
        /// </summary>

        /// <summary>
        /// Defines the plc_type.
        /// </summary>
        internal string plc_type = ConfigurationManager.AppSettings["DEFAULT_PLC_TYPE"];

        /// <summary>
        /// Defines the plc.
        /// </summary>
        public Plc plc;

        public int address_start;
        public string address_tmp;
        public int address_tmp_int;
        public int address_end;
        public string last_var_data_type;
        public int last_var_data_size;
        public string read_tmp;
        internal byte[] event_data_val;
        public string current_l2_id;

        string file_path;
        string json_string;
        dynamic event_data;
        public DataDisplayMainControl()
        {
            InitializeComponent();
            sample_interval_nud.Value = 5000;
            panel1.AutoScroll = true;
            tracking_timer.Enabled = false;
        }
        /*
        public static bool IsPropertyExist(dynamic evn_data, string name)
        {
            if (evn_data is ExpandoObject)
                return ((IDictionary<string, object>)evn_data).ContainsKey(name);

            return evn_data.GetType().GetProperty(name) != null;
        }*/


        public void Fill_event_selection(string file_path)
        {
            this.file_path = file_path;
            try
            {
                
                json_string = File.ReadAllText(file_path);
                event_data = JsonConvert.DeserializeObject(json_string);
                foreach (var particular_event in event_data.events)
                {
                    if (((string)particular_event.Value.plc_type).Contains(plc_type))
                    {
                        try
                        {
                            string tmp = particular_event.Value.evn_data.Input_Coil_L2_Identifier.format;
                            if (tmp == null)
                                continue;
                            CompactEventDisplay new_evn_display = new CompactEventDisplay(particular_event.Name);
                            new_evn_display.Visible = true;
                            new_evn_display.BringToFront();
                            panel1.Controls.Add(new_evn_display);
                            
                        }
                        catch (RuntimeBinderException)
                        {
                            //  Property doesn't exist
                        }
                    }
                        
                }
            }
            catch (Exception)
            {

                throw;
            }
        }

        private void start_tracking_btn_Click_1(object sender, System.EventArgs e)
        {
            tracking_timer.Interval = (int)sample_interval_nud.Value;
            if (tracking_timer.Enabled == true)
            {
                tracking_timer.Enabled = false;
                start_tracking_btn.Text = "Start tracking";
            }
            else
            {
                tracking_timer.Enabled = true;
                start_tracking_btn.Text = "Stop tracking";
                foreach (CompactEventDisplay compact_control in panel1.Controls)
                {
                    compact_control.time_of_arrival_val.Text = "N/A";
                    compact_control.evn_details = String.Empty;
                }
            }
        }
        private string Get_variable_val(dynamic evn_var, byte[] event_data_val)
        {
            string return_str = string.Empty;
            byte[] val_array = new byte[evn_var.Value.size];
            byte[] num_val_array = new byte[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"])];

            int var_address = Int32.Parse(Regex.Match((string)evn_var.Value.address, @"\d+").Value);
            switch ((string)evn_var.Value.format)
            {
                case "DI":
                    for (int i = 0; i < Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]); i++)
                    {
                        num_val_array[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]) - 1 - i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.BitConverter.ToInt32(num_val_array, 0).ToString();
                    break;
                case "MS":
                    for (int i = 0; i < (int)evn_var.Value.size; i++)
                    {
                        val_array[i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.Text.Encoding.Default.GetString(val_array);
                    break;
                case "FP":
                    for (int i = 0; i < Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]); i++)
                    {
                        num_val_array[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]) - 1 - i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.BitConverter.ToSingle(num_val_array, 0).ToString();
                    break;

                default:
                    break;
            }

            return return_str.Trim();
        }
        private void tracking_timer_Tick(object sender, EventArgs e)
        {
            foreach(CompactEventDisplay compact_control in panel1.Controls)
            {
                foreach (dynamic particular_event in event_data.events)
                {
                    if(compact_control.event_name_val.Text == particular_event.Name) 
                    {
                        address_start = -1;
                        foreach (dynamic evn_var in particular_event.Value.evn_data)
                        {
                            address_tmp = evn_var.Value.address;
                            address_tmp = Regex.Match(address_tmp, @"\d+").Value;
                            address_tmp_int = Int32.Parse(address_tmp);
                            if (address_start == -1)
                            {
                                address_start = address_tmp_int;
                            }
                            address_end = address_tmp_int;
                            last_var_data_type = evn_var.Value.format;
                            last_var_data_size = evn_var.Value.size;
                        }

                        switch (last_var_data_type)
                        {
                            case "DI":
                                address_end += Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]);
                                break;
                            case "MS":
                                address_end += last_var_data_size;
                                break;
                            case "FP":
                                address_end += Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]);
                                break;
                            default:
                                break;
                        }

                        read_tmp += "\n\nEvent name: " + particular_event.Name + "\nVariables: ";
                        event_data_val = plc.ReadBytes(DataType.DataBlock, (int)particular_event.Value.db_no, address_start, address_end);
                        foreach (var evn_var in particular_event.Value.evn_data)
                        {
                            string variable_val = Get_variable_val(evn_var, event_data_val);
                            if (evn_var.Name == "Input_Coil_L2_Identifier")
                                current_l2_id = variable_val;
                            read_tmp += "\n" + evn_var.Name;
                            read_tmp += " : " + variable_val;
                            if (evn_var.Value.min_val != 0 || evn_var.Value.max_val != 0)
                            {
                                if (Convert.ToSingle(variable_val) < (float)evn_var.Value.min_val || Convert.ToSingle(variable_val) > (float)evn_var.Value.max_val)
                                {
                                    read_tmp += "   - VALUE OUT OF BOUNDS";
                                }
                            }
                            foreach (CompactEventDisplay evn_display in panel1.Controls)
                            {
                                if (evn_display.event_name_val.Text == particular_event.Name)
                                {
                                    if (coil_id_textbox.Text == current_l2_id) { 
                                        evn_display.time_of_arrival_val.Text = DateTime.Now.ToString("HH:mm:ss");
                                        evn_display.evn_details = read_tmp;
                                    }
                                }
                            }                            
                        }
                        read_tmp = string.Empty;
                    }
                }
            }
        }        
    }
}
