﻿namespace S7_PLC_Monitoring
{
    using Newtonsoft.Json;
    using S7.Net;
    using System;
    using System.Data;
    using System.IO;
    using System.Linq;
    using System.Text.RegularExpressions;
    using System.Windows.Forms;

    /// <summary>
    /// Defines the <see cref="DataDisplaySecondaryControl" />.
    /// </summary>
    public partial class DataDisplaySecondaryControl : UserControl
    {
        /// <summary>
        /// Defines the json_string.
        /// </summary>
        internal string json_string;

        /// <summary>
        /// Defines the event_data.
        /// </summary>
        internal dynamic event_data;

        /// <summary>
        /// Defines the plc_type.
        /// </summary>
        internal string plc_type = "SPEED MASTER PLC";

        /// <summary>
        /// Gets or sets the plc.
        /// </summary>
        public Plc plc { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="DataDisplaySecondaryControl"/> class.
        /// </summary>
        public DataDisplaySecondaryControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// The Fill_event_selection.
        /// </summary>
        /// <param name="file_path">The file_path<see cref="string"/>.</param>
        public void Fill_event_selection(string file_path)
        {
            try
            {
                event_select_cb.Items.Clear();
                json_string = File.ReadAllText(file_path);
                event_data = JsonConvert.DeserializeObject(json_string);
                foreach (var particular_event in event_data.events)
                {
                    if (((string)particular_event.Value.plc_type).Contains(plc_type))
                        this.event_select_cb.Items.Add(particular_event.Name);
                }
            }
            catch (Exception)
            {

                throw;
            }
        }

        /// <summary>
        /// The read_value_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void read_value_btn_Click(object sender, EventArgs e)
        {
            if (this.event_select_cb.Text != string.Empty && this.variable_select_cbox.Text != string.Empty)
            {

                foreach (var particular_event in event_data.events)
                {
                    if (particular_event.Name == this.event_select_cb.SelectedItem.ToString())
                    {
                        foreach (var evn_var in particular_event.Value.evn_data)
                        {
                            if (evn_var.Name == this.variable_select_cbox.SelectedItem.ToString())
                            {
                                read_plc_val(evn_var, particular_event);
                                break;
                            }

                        }
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// The read_plc_val.
        /// </summary>
        /// <param name="evn_var">The evn_var<see cref="dynamic"/>.</param>
        /// <param name="evn">The evn<see cref="dynamic"/>.</param>
        private void read_plc_val(dynamic evn_var, dynamic evn)
        {
            string address = evn_var.Value.address;
            address = string.Concat(address.Where(c => !Char.IsWhiteSpace(c)));
            int address_int = Int32.Parse(Regex.Match(address, @"\d+").Value);
            string data_block = "DB" + evn.Value.db_no;
            int db_no_int = (int)evn.Value.db_no;
            string data_type = evn_var.Value.format;
            int min_val = evn_var.Value.min_val;
            int max_val = evn_var.Value.max_val;
            string read_address = data_block + "." + address;
            int size = evn_var.Value.size;
            try
            {
                this.error_label.Hide();
                switch (data_type)
                {
                    case "DI":
                        this.value_text_box.Text = ((uint)plc.Read(read_address)).ConvertToInt().ToString();
                        break;
                    case "MS":
                        byte[] tempString = (byte[])plc.Read(DataType.DataBlock, db_no_int, address_int, VarType.Byte, size);
                        this.value_text_box.Text = System.Text.Encoding.Default.GetString(tempString);
                        break;
                    case "FP":
                        this.value_text_box.Text = ((uint)plc.Read(read_address)).ConvertToFloat().ToString();
                        break;

                    default:
                        break;
                }
                if (min_val != 0 || max_val != 0)
                {
                    if (Convert.ToSingle(value_text_box.Text) < (float)evn_var.Value.min_val || Convert.ToSingle(value_text_box.Text) > (float)evn_var.Value.max_val)
                    {
                        value_text_box.Text += "   - VALUE OUT OF BOUNDS";
                    }
                }
            }
            catch (Exception)
            {
                this.error_label.Show();
            }
        }

        /// <summary>
        /// The event_select_cb_SelectedIndexChanged.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void event_select_cb_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (event_select_cb.SelectedItem != null)
            {

                this.variable_select_cbox.Items.Clear();
                foreach (var particular_event in event_data.events)
                {
                    if (particular_event.Name == this.event_select_cb.SelectedItem.ToString())
                    {
                        foreach (var evn_var in particular_event.Value.evn_data)
                        {
                            this.variable_select_cbox.Items.Add(evn_var.Name);
                        }
                        break;
                    }
                }
                this.variable_select_cbox.SelectedIndex = 0;
            }
        }
    }
}
