﻿namespace S7_PLC_Monitoring
{
    using Newtonsoft.Json;
    using S7.Net;
    using System;
    using System.Data;
    using System.IO;
    using System.Linq;
    using System.Text.RegularExpressions;
    using System.Windows.Forms;
    using System.Configuration;

    /// <summary>
    /// Defines the <see cref="MonitoringControl" />.
    /// </summary>
    public partial class MonitoringControl : UserControl
    {
        /// <summary>
        /// Defines the json_string.
        /// </summary>
        internal string json_string;

        /// <summary>
        /// Defines the file_path.
        /// </summary>
        internal string file_path;

        /// <summary>
        /// Defines the event_data.
        /// </summary>
        internal dynamic event_data;

        /// <summary>
        /// Defines the address_tmp.
        /// </summary>
        internal string address_tmp;

        /// <summary>
        /// Defines the last_data_type.
        /// </summary>
        internal string last_var_data_type;

        /// <summary>
        /// Defines the last_var_data_size.
        /// </summary>
        internal int last_var_data_size;

        /// <summary>
        /// Defines the address_tmp.
        /// </summary>
        internal int address_tmp_int;

        /// <summary>
        /// Defines the address_min.
        /// </summary>
        internal int address_start;

        /// <summary>
        /// Defines the address_end.
        /// </summary>
        internal int address_end;

        /// <summary>
        /// Gets or sets the plc.
        /// </summary>
        public Plc plc { get; set; }

        /// <summary>
        /// Defines the result_addr.
        /// </summary>
        internal string result_addr;

        /// <summary>
        /// Defines the event_data_val.
        /// </summary>
        internal byte[] event_data_val;

        /// <summary>
        /// Defines the event_data_val.
        /// </summary>
        internal dynamic selected_event;

        /// <summary>
        /// Defines the plc_type.
        /// </summary>
        internal string plc_type = "SPEED MASTER PLC";

        /// <summary>
        /// Defines the lb_flag.
        /// </summary>
        internal int lb_flag = 0;

        /// <summary>
        /// Initializes a new instance of the <see cref="MonitoringControl"/> class.
        /// </summary>
        public MonitoringControl()
        {
            InitializeComponent();
            this.error_label.Hide();
            start_listening_btn.Enabled = false;
            stop_listening_btn.Enabled = false;
            sample_interval_nud.Value = 500;
            sampling_timer.Interval = (int)sample_interval_nud.Value;
            sampling_timer.Enabled = false;
            event_select_lb.Show();
            event_select_lb.BringToFront();
            variable_select_lb.Hide();
            variable_select_lb.SendToBack();
            variable_display_panel.FlowDirection = FlowDirection.BottomUp;
            variable_display_panel.AutoScroll = true;
            variable_display_panel.WrapContents = false;
        }

        /// <summary>
        /// The read_plc_val.
        /// </summary>
        /// <param name="evn_var">The evn_var<see cref="dynamic"/>.</param>
        /// <param name="evn">The evn<see cref="dynamic"/>.</param>
        private void read_plc_val(dynamic evn_var, dynamic evn)
        {
            try
            {
                string address = evn_var.Value.address;
                address = string.Concat(address.Where(c => !Char.IsWhiteSpace(c)));
                int address_int = Int32.Parse(Regex.Match(address, @"\d+").Value);
                string data_block = "DB" + evn.Value.db_no;
                int db_no_int = (int)evn.Value.db_no;
                string data_type = evn_var.Value.format;
                int min_val = evn_var.Value.min_val;
                int max_val = evn_var.Value.max_val;
                string read_address = data_block + "." + address;
                int size = evn_var.Value.size;
                string read_tmp = "Variable name: " + evn_var.Name + "\nValue: ";

                this.error_label.Hide();
                switch (data_type)
                {
                    case "DI":
                        read_tmp += ((uint)plc.Read(read_address)).ConvertToInt().ToString();

                        break;
                    case "MS":
                        byte[] tempString = (byte[])plc.Read(DataType.DataBlock, db_no_int, address_int, VarType.Byte, size);
                        read_tmp += System.Text.Encoding.Default.GetString(tempString);
                        break;
                    case "FP":
                        read_tmp += ((uint)plc.Read(read_address)).ConvertToFloat().ToString();
                        break;

                    default:
                        break;
                }

                read_tmp += "\nTime: " + DateTime.Now.ToString("HH:mm:ss.ff") + "\n";
                if (checkBox1.Checked)
                {
                    File.AppendAllText(result_addr, read_tmp);
                }
            }
            catch (Exception)
            {
                this.error_label.Show();
            }
        }

        /// <summary>
        /// The Fill_event_selection.
        /// </summary>
        /// <param name="file_path">The file_path<see cref="string"/>.</param>
        public void Fill_event_selection(string file_path)
        {
            this.file_path = file_path;
            try
            {
                event_select_lb.Items.Clear();
                json_string = File.ReadAllText(file_path);
                event_data = JsonConvert.DeserializeObject(json_string);
                foreach (var particular_event in event_data.events)
                {
                    if (((string)particular_event.Value.plc_type).Contains(plc_type))
                        this.event_select_lb.Items.Add(particular_event.Name);
                }
            }
            catch (Exception)
            {

                throw;
            }
        }

        /// <summary>
        /// The timer1_Tick.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            sampling_timer.Enabled = false;
            string read_tmp = "\n<------------------------------------------>\nTime: " + DateTime.Now.ToString("HH:mm:ss.ff");
            foreach (string particular_item in event_select_lb.SelectedItems)
            {
                foreach (dynamic particular_event in event_data.events)
                {

                    if (particular_event.Name == particular_item)
                    {
                        try
                        {

                            address_start = -1;
                            foreach (dynamic evn_var in particular_event.Value.evn_data)
                            {
                                address_tmp = evn_var.Value.address;
                                address_tmp = Regex.Match(address_tmp, @"\d+").Value;
                                address_tmp_int = Int32.Parse(address_tmp);
                                if (address_start == -1)
                                {
                                    address_start = address_tmp_int;
                                }
                                address_end = address_tmp_int;
                                last_var_data_type = evn_var.Value.format;
                                last_var_data_size = evn_var.Value.size;
                            }

                            switch (last_var_data_type)
                            {
                                case "DI":
                                    address_end += Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]);
                                    break;
                                case "MS":
                                    address_end += last_var_data_size;
                                    break;
                                case "FP":
                                    address_end += Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]);
                                    break;
                                default:
                                    break;
                            }
                            error_label.Hide();

                            read_tmp += "\n\nEvent name: " + particular_event.Name + "\nVariables: ";
                            event_data_val = plc.ReadBytes(DataType.DataBlock, (int)particular_event.Value.db_no, address_start, address_end);
                            if (variable_select_lb.SelectedItem == null)
                            {
                                foreach (var evn_var in particular_event.Value.evn_data)
                                {
                                    string variable_val = Get_variable_val(evn_var, event_data_val);
                                    read_tmp += "\n" + evn_var.Name;
                                    read_tmp += " : " + variable_val;
                                    if (evn_var.Value.min_val != 0 || evn_var.Value.max_val != 0)
                                    {
                                        if (Convert.ToSingle(variable_val) < (float)evn_var.Value.min_val || Convert.ToSingle(variable_val) > (float)evn_var.Value.max_val)
                                        {
                                            read_tmp += "   - VALUE OUT OF BOUNDS";
                                        }
                                    }
                                }
                            }
                            else
                            {
                                foreach (var evn_var in particular_event.Value.evn_data)
                                {
                                    if (variable_select_lb.SelectedItems.Contains(particular_event.Name + " - " + evn_var.Name))
                                    {
                                        string variable_val = Get_variable_val(evn_var, event_data_val);
                                        SingleVariableDisplayControl tmp_control = (SingleVariableDisplayControl)(variable_display_panel.Controls.Find(particular_event.Name + " - " + evn_var.Name, true)[0]);
                                        tmp_control.variable_value_label.Text = variable_val;
                                        read_tmp += "\n" + evn_var.Name;
                                        read_tmp += " : " + variable_val;
                                        if (evn_var.Value.min_val != 0 || evn_var.Value.max_val != 0)
                                        {
                                            if (Convert.ToSingle(variable_val) < (float)evn_var.Value.min_val || Convert.ToSingle(variable_val) > (float)evn_var.Value.max_val)
                                            {
                                                tmp_control.variable_value_label.ForeColor = System.Drawing.Color.Red;
                                                read_tmp += "   - VALUE OUT OF BOUNDS";
                                            }
                                        }
                                    }
                                }
                            }
                           
                            if (checkBox1.Checked)
                            {
                                File.AppendAllText(result_addr, read_tmp);
                                read_tmp = "\n";
                            }
                        }
                        catch (Exception)
                        {
                            sampling_timer.Enabled = false;
                            start_listening_btn.Enabled = true;
                            stop_listening_btn.Enabled = false;
                            error_label.Show();

                        }
                        break;
                    }
                }

            }
            sampling_timer.Enabled = true;
        }

        private string Get_variable_val(dynamic evn_var, byte[] event_data_val)
        {
            string return_str = string.Empty;
            byte[] val_array = new byte[evn_var.Value.size];
            byte[] num_val_array = new byte[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"])];

            int var_address = Int32.Parse(Regex.Match((string)evn_var.Value.address, @"\d+").Value);
            switch ((string)evn_var.Value.format)
            {
                case "DI":
                    for (int i = 0; i < Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]); i++)
                    {
                        num_val_array[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]) - 1 - i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.BitConverter.ToInt32(num_val_array, 0).ToString();
                    break;
                case "MS":
                    for (int i = 0; i < (int)evn_var.Value.size; i++)
                    {
                        val_array[i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.Text.Encoding.Default.GetString(val_array);
                    break;
                case "FP":
                    for (int i = 0; i < Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]); i++)
                    {
                        num_val_array[Int32.Parse(ConfigurationManager.AppSettings["NUM_VALUE_LENGTH"]) - 1 - i] = event_data_val[var_address - address_start + i];
                    }
                    return_str = System.BitConverter.ToSingle(num_val_array, 0).ToString();
                    break;

                default:
                    break;
            }

            return return_str.Trim(); 
        }

        /// <summary>
        /// The start_listening_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void start_listening_btn_Click(object sender, EventArgs e)
        {
            variable_display_panel.Controls.Clear();
            sampling_timer.Interval = (int)sample_interval_nud.Value;
            sampling_timer.Enabled = true;
            stop_listening_btn.Enabled = true;
            start_listening_btn.Enabled = false;
            variable_select_lb.Enabled = false;
            event_select_lb.Enabled = false;
            
                
            foreach (string selected_variable in variable_select_lb.SelectedItems)
            {
                SingleVariableDisplayControl new_variable = new SingleVariableDisplayControl(selected_variable);
                new_variable.Name = selected_variable;
                variable_display_panel.Controls.Add(new_variable);
                new_variable.Show();
                new_variable.BringToFront();
            } 

        }

        /// <summary>
        /// The stop_listening_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void stop_listening_btn_Click(object sender, EventArgs e)
        {
            sampling_timer.Enabled = false;
            start_listening_btn.Enabled = true;
            stop_listening_btn.Enabled = false;
            variable_select_lb.Enabled = true;
            event_select_lb.Enabled = true;
        }

        

        private void event_variable_select_switch_Click(object sender, EventArgs e)
        {
            if(lb_flag == 0)
            {
                event_select_lb.Hide();
                event_select_lb.SendToBack();
                variable_select_lb.Show();
                variable_select_lb.BringToFront();
                lb_flag = 1;
                event_variable_select_switch.Text = "Select events";
            }
            else
            {
                variable_select_lb.Hide();
                variable_select_lb.SendToBack();
                event_select_lb.Show();
                event_select_lb.BringToFront();
                lb_flag = 0;
                event_variable_select_switch.Text = "Select variables";
            }
        }
        /// <summary>
        /// The event_select_lb_SelectedIndexChanged.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>

        private void event_select_lb_SelectedIndexChanged(object sender, EventArgs e)
        {

            sampling_timer.Enabled = false;
            result_addr = System.IO.Directory.GetParent(System.IO.Directory.GetParent(System.IO.Directory.GetParent(Environment.CurrentDirectory).FullName).FullName) + "\\LOG_files\\" + DateTime.Now.ToString("MM-dd-yyyy");
            result_addr += ".txt";
            stop_listening_btn.Enabled = false;
            address_start = -1;
            address_end = -1;
            selected_event = null;
            string evn_name = "";
            variable_select_lb.Items.Clear();
            if (this.event_select_lb.Text != string.Empty)
            {
                start_listening_btn.Enabled = true;
                foreach (var particular_event in event_data.events)
                {
                    foreach (string selected_event_name in event_select_lb.SelectedItems)
                    {
                        if (particular_event.Name == selected_event_name)
                        {
                            selected_event = particular_event;
                            foreach (var evn_var in selected_event.Value.evn_data)
                            {
                                evn_name = selected_event.Name;
                                if (event_select_lb.SelectedItems.Count > 1)
                                    evn_name = selected_event.Name + " - ";
                                else
                                    evn_name = "";
                                variable_select_lb.Items.Add(evn_name + evn_var.Name);
                                
                            }
                        }

                    }
                }

                
            }


        }
    }
}
