﻿namespace S7_PLC_Monitoring
{
    using Newtonsoft.Json;
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Data.SqlClient;
    using System.IO;
    using System.Text.RegularExpressions;
    using System.Windows.Forms;
    using Excel = Microsoft.Office.Interop.Excel;

    /// <summary>
    /// Defines the <see cref="ParseNewXLSFile" />.
    /// </summary>
    public partial class ParseNewXLSFile : UserControl
    {
        

        /// <summary>
        /// Defines the current event size. Used for checking if the event is loaded
        /// correctly
        /// </summary>
        public int current_evn_size;

        /// <summary>
        /// Defines the rows with data lenth errors.
        /// </summary>
        public List<string> error_rows = new List<string>();

        /// <summary>
        /// Defines the error_flag.
        /// </summary>
        public int error_flag = 0;

        /// <summary>
        /// Defines the struct_array_mult.
        /// </summary>
        public int struct_array_mult = 1;

        /// <summary>
        /// Defines the <see cref="event_characteristics" />.
        /// </summary>
        public struct event_characteristics
        {
            

            /// <summary>
            /// Defines the name.
            /// </summary>
            public string name;

            /// <summary>
            /// Defines the evn_no.
            /// </summary>
            public int evn_no;

            /// <summary>
            /// Defines the db_no.
            /// </summary>
            public int db_no;

            /// <summary>
            /// Defines the dl.
            /// </summary>
            public int dl;

            /// <summary>
            /// Defines the cycle.
            /// </summary>
            public int cycle;

            /// <summary>
            /// Defines the plc_type.
            /// </summary>
            public string plc_type;

            
        }

        /// <summary>
        /// Defines the file_path.
        /// </summary>
        internal string file_path = string.Empty;


        /// <summary>
        /// DB connection meant for loading min and max values from db insted of xls
        /// - not used
        /// </summary>


        /// <summary>
        /// Initializes a new instance of the <see cref="ParseNewXLSFile"/> class.
        /// </summary>
        public ParseNewXLSFile()
        {
            InitializeComponent();
            selected_file_label.Hide();
            selected_file_val_label.Hide();
            //db_connection.Open();//   - not used 
            Limits_from_db_checkbox.Checked = false;
            Limits_from_db_checkbox.Enabled = false;//loading from db 
            Limits_from_db_checkbox.Visible = false;//not yet implemented
        }

        /// <summary>
        /// The data_buffer_end_conditions.
        /// </summary>
        /// <param name="var_name">The var_name<see cref="string"/>.</param>
        /// <param name="var_addres">The var_addres<see cref="string"/>.</param>
        /// <param name="var_note">The var_note<see cref="string"/>.</param>
        /// <param name="na">The na<see cref="string"/>.</param>
        /// <returns>The <see cref="Boolean"/>.</returns>
        public Boolean data_buffer_end_conditions(string var_name, string var_addres, string var_note, string na)
        {
            if (var_name.Contains("STRUCT"))
                return false;
            if (var_addres == string.Empty)
                return false;
            if (var_note != null)
            {
                if (var_note.Contains("NOT USED"))
                    return false;
            }
            if (na != null)
            {
                if (na.Contains("X"))
                    return false;
            }

            return true;
        }

        /// <summary>
        /// The select_file_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void select_file_btn_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.InitialDirectory = Environment.CurrentDirectory;
                openFileDialog.Filter = "Excel files (*XLS)|*.XLS|All files (*.*)|*.*";
                openFileDialog.FilterIndex = 2;
                openFileDialog.RestoreDirectory = true;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    //Get the path of specified file
                    file_path = openFileDialog.FileName;

                }
            }
            if (file_path != string.Empty)
            {

                selected_file_label.Show();
                selected_file_val_label.Text = file_path;
                selected_file_val_label.Show();
            }
        }

        /// <summary>
        /// The Confirm_btn_Click.
        /// </summary>
        /// <param name="sender">The sender<see cref="object"/>.</param>
        /// <param name="e">The e<see cref="EventArgs"/>.</param>
        private void Confirm_btn_Click(object sender, EventArgs e)
        {
            error_flag = 0; 
            this.error_label.Hide();
            this.success_label.Hide();
            this.loading_label.Show();
            Loaded_items_label.Hide();
            error_rows_label.Hide();
            Loaded_items_tw.Hide();
            Excel.Workbook WB = null;
            Excel.Application xls_source = new Excel.Application();
            string xls_address = file_path;
            WB = xls_source.Workbooks.Open(xls_address);
            try
            {

                

                Excel.Worksheet TOC_worksheet = (Excel.Worksheet)WB.Worksheets["Table_of_Contents"];
                Excel.Worksheet curr_worksheet = null;

                int curr_row = 6;       //6 is the start row of message definitions 
                string tmp_string;

                string result_addr = System.IO.Directory.GetParent(System.IO.Directory.GetParent(System.IO.Directory.GetParent(Environment.CurrentDirectory).FullName).FullName) + "\\JSON_files\\" + WB.Name + ".json";
                File.WriteAllText(result_addr, String.Empty);

                string json_buffer = "{\"Workbook_name\" :\"" + WB.Name + "\",\n\"events\" : {";

                event_characteristics current_event;

                while (TOC_worksheet.Cells[curr_row, 1].Value != null)
                {
                    loading_label.Text = "Loading: " + TOC_worksheet.Cells[curr_row, 1].Value;
                    Excel.Range curr_row_range = (Excel.Range)TOC_worksheet.Rows[curr_row];
                    if (!curr_row_range.Hidden)
                    {

                        current_event.name = TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_NAME_COL"])].Value;

                        tmp_string = TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_NO_COL"])].Value;
                        current_event.evn_no = Int32.Parse(Regex.Match(tmp_string, @"\d+").Value);

                        tmp_string = TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_DB_NO_COL"])].Value;
                        current_event.db_no = Int32.Parse(Regex.Match(tmp_string, @"\d+").Value);

                        current_event.dl = (int)TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_DATA_LEN_COL"])].Value;

                        if (TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_CYCLE_COL"])].Value is double)
                            current_event.cycle = (int)TOC_worksheet.Cells[curr_row, 8].Value;
                        else
                            current_event.cycle = -1;

                        current_event.plc_type = TOC_worksheet.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_PLC_TYPE_COL"])].Value;

                        json_buffer += "\n\t\"" + current_event.name + "\" : {";
                        json_buffer += "\n\t\"evn_no\" : " + current_event.evn_no;
                        json_buffer += ",\n\t\"db_no\" : " + current_event.db_no;
                        json_buffer += ",\n\t\"dl\" : " + current_event.dl;
                        json_buffer += ",\n\t\"cycle\" : " + current_event.cycle;
                        json_buffer += ",\n\t\"plc_type\" : \"" + current_event.plc_type + "\"";
                        json_buffer += ",\n\t\"evn_data\" : {";

                        curr_worksheet = (Excel.Worksheet)WB.Worksheets[current_event.name];
                        json_buffer += parse_evn_data(curr_worksheet);

                        json_buffer += "\n\t\t}\n\t},";
                    }
                    {
                        if (TOC_worksheet.Cells[curr_row + 1, 1].Value == null)
                            json_buffer = json_buffer.Remove(json_buffer.Length - 1);

                        File.AppendAllText(result_addr, json_buffer + Environment.NewLine);
                        json_buffer = string.Empty;
                    }
                    curr_row++;
                }
                File.AppendAllText(result_addr, "}\n}" + Environment.NewLine);
                this.success_label.BringToFront();
                this.success_label.Show();
                this.loading_label.Hide();
                string json_string = File.ReadAllText(result_addr);
                dynamic event_data = JsonConvert.DeserializeObject(json_string);
                foreach (var particular_event in event_data.events)
                {
                    Loaded_items_tw.Nodes.Add(particular_event.Name);
                }
                
                for (int i = 0; i < Loaded_items_tw.Nodes.Count; i++)
                {
                    foreach (var particular_event in event_data.events)
                    {
                        if(particular_event.Name == Loaded_items_tw.Nodes[i].Text)
                                
                            foreach(var particular_var in particular_event.Value.evn_data)
                                Loaded_items_tw.Nodes[i].Nodes.Add(particular_var.Name);

                    }
                }
                Loaded_items_label.Show();
                Loaded_items_tw.Show(); 
                Loaded_items_label.BringToFront();
                Loaded_items_tw.BringToFront();
                selected_file_val_label.Hide();
                if(error_flag == 1)
                {
                    
                    error_rows_label.Show();
                }
            }
            catch (Exception)
            {
                this.error_label.BringToFront();
                this.error_label.Show();
                file_path = string.Empty;
            }
            finally
            {
                WB.Close();
            }
        }

        /// <summary>   
        /// The parse_evn_data.
        /// </summary>
        /// <param name="ws">The ws<see cref="Excel.Worksheet"/>.</param>
        /// <returns>The <see cref="string"/>.</returns>
        private string parse_evn_data(Excel.Worksheet ws)
        {
            int curr_row = 1;
            string json_buffer = string.Empty;
            int tmp_int;
           
            int expected_evn_size;
            while (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value != "DATA BUFFER DATA STRUCTURE" && ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value != "CIRCULAR BUFFER DATA STRUCTURE" && ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value != "LIST BUFFER DATA STRUCTURE" && ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value != "DATA BUFFER DATA STRUCTURE")
            {   
                curr_row++;
            }
            expected_evn_size = (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EXPECTED_EVN_SIZE_COL"])].Value2;
            curr_row++;
            current_evn_size = 0;
            while (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value != null)
            {
                if (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value == "CIRCULAR BUFFER FOOTER")
                    break;
                if (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_VAR_SIZE_COL"])].Value != null && !((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value).Contains("STRUCT"))
                    current_evn_size += (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_VAR_SIZE_COL"])].Value2 * struct_array_mult;
                else if (((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value).Contains("STRUCT"))
                {
                    if(ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_STRING_LEN_COL"])].Value != null)
                    {
                        struct_array_mult = (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_STRING_LEN_COL"])].Value;
                    }
                    else
                    {
                        struct_array_mult = 1;
                    }
                }    
                if (data_buffer_end_conditions(((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value), ((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_ADDRESS_COL"])].Value), ((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_REMARKS_COL"])].Value), ((string)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_NA_COL"])].Value)))
                {
                    json_buffer += "\n\t\t\t\"" + ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_DESCRIPTION_COL"])].Value.Replace(" ", "_") + "\" : {";
                    json_buffer += "\n\t\t\t\"address\" : \"" + ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_ADDRESS_COL"])].Value + "\"";
                    json_buffer += ",\n\t\t\t\"format\" : \"" + ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_EVN_FROMAT_COL"])].Value + "\"";

                    if (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_STRING_LEN_COL"])].Value != null)    
                        tmp_int = (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_STRING_LEN_COL"])].Value;
                    else
                        tmp_int = 0;
                    json_buffer += ",\n\t\t\t\"size\" : " + tmp_int;
                    if (Limits_from_db_checkbox.Checked)
                    {
                        //TODO - implement the loading of limits from the db
                    }
                        
                    else if (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_MIN_VAL_COL"])].Value != null/* && !Limits_from_db_checkbox.Checked*/)
                                tmp_int = (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_MIN_VAL_COL"])].Value;
          
                    else
                        tmp_int = 0;
                    json_buffer += ",\n\t\t\t\"min_val\" : " + tmp_int;
                    if (ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_MAX_VAL_COL"])].Value != null/* && !Limits_from_db_checkbox.Checked*/)
                        tmp_int = (int)ws.Cells[curr_row, Int32.Parse(ConfigurationManager.AppSettings["XLS_MAX_VAL_COL"])].Value;
                    else
                        tmp_int = 0;
                    json_buffer += ",\n\t\t\t\"max_val\" : " + tmp_int;
                
                    json_buffer += "\n\t\t\t},";
                }
                curr_row++;
            }
            if(expected_evn_size != current_evn_size)
            {
                error_flag = 1;
                error_rows_label.Text += "\n\n" + ws.Name.ToString() + "\nexpected size: " + expected_evn_size + "\nactual size: " + current_evn_size;
            }
            json_buffer = json_buffer.Remove(json_buffer.Length - 1);
            return json_buffer;
        }
    }
}
